﻿#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import print_function

import base64
import json
from geolite2 import geolite2
from user_agents import parse

print('Loading Lambda function')


def lambda_handler(event, context):
    output = []
    reader = geolite2.reader()
    for record in event['records']:
        payload = base64.b64decode(record['data'])
        payloadstr = str(payload.decode('utf-8'))
        print('Incoming Record from Kinesis Firehose : ' + payloadstr)
        json_data = json.loads(payloadstr)
        user_agent = json_data['agent']
        if user_agent != 'ELB-HealthChecker/2.0':
            ip_addr = json_data['host']
            geo_loc = reader.get(ip_addr)
            try:
                geoip_city = geo_loc['city']['names']['en']
            except:
                geoip_city = 'Unidentified'
            try:
                geoip_country = geo_loc['country']['names']['en']
            except:
                geoip_country = 'Unidentified'
            try:
                geoip_lat = str(geo_loc['location']['latitude'])
                geoip_lon = str(geo_loc['location']['longitude'])
            except:
                geoip_lat = '0'
                geoip_lon = '0'
            try:
                user_agent = parse(user_agent)
                parsed_browser = user_agent.browser.family
                parsed_os = user_agent.os.family
            except:
                parsed_browser = 'Unidentified'
                parsed_os = 'Unidentified'
            try:
                request_str = json_data['request']
                webpage = request_str[request_str.find('/')
                    + 1:request_str.find('php', request_str.find('/'))
                    - 1]
            except:
                webpage = 'None'
            referer_str = json_data['referer']
            refering_link = 'None'
            if referer_str != '-':
                try:
                    refering_link = referer_str[referer_str.find('/',
                            8) + 1:referer_str.find('php',
                            referer_str.find('/')) - 1]
                except:
                    refering_link = 'None'
            final_str = payloadstr[0:int(len(payloadstr)) - 2]
            append_string = ',"webpage":"' + webpage + '"'
            final_str = final_str + append_string
            append_string = ',"refering_page":"' + refering_link + '"'
            final_str = final_str + append_string
            append_string = ',"city":"' + geoip_city + '"'
            final_str = final_str + append_string
            append_string = ',"country":"' + geoip_country + '"'
            final_str = final_str + append_string
            append_string = ',"browser":"' + parsed_browser + '"'
            final_str = final_str + append_string
            append_string = ',"os":"' + parsed_os + '"'
            final_str = final_str + append_string
            append_string = ',"location":{"lat":' + geoip_lat \
                + ',"lon":' + geoip_lon + '}}\n'
            final_str = final_str + append_string
            print('Transformed Record going back to Kinesis Firehose : '
                  + str(final_str))
            output_record = {'recordId': record['recordId'],
                             'result': 'Ok',
                             'data': base64.b64encode(final_str.encode()).decode()}
        else:
            output_record = {'recordId': record['recordId'],
                             'result': 'Dropped', 'data': record['data'
                             ]}
            print('Dropping this record since it is from Load Balancer Healthchecks : '
                   + str(output_record))
        output.append(output_record)

    print('Successfully processed {} records.'.format(len(event['records'
          ])))
    return {'records': output}
